/*******************************************************************************
* Freescale Semiconductor Inc.
* (c) Copyright 2011-2012 Freescale Semiconductor, Inc.
* ALL RIGHTS RESERVED.
********************************************************************************
Services performed by FREESCALE in this matter are performed AS IS and without
any warranty. CUSTOMER retains the final decision relative to the total design
and functionality of the end product. FREESCALE neither guarantees nor will be
held liable by CUSTOMER for the success of this project.
FREESCALE DISCLAIMS ALL WARRANTIES, EXPRESSED, IMPLIED OR STATUTORY INCLUDING,
BUT NOT LIMITED TO, IMPLIED WARRANTY OF MERCHANTABILITY OR FITNESS FOR
A PARTICULAR PURPOSE ON ANY HARDWARE, SOFTWARE ORE ADVISE SUPPLIED 
TO THE PROJECT BY FREESCALE, AND OR NAY PRODUCT RESULTING FROM FREESCALE 
SERVICES. IN NO EVENT SHALL FREESCALE BE LIABLE FOR INCIDENTAL OR CONSEQUENTIAL 
DAMAGES ARISING OUT OF THIS AGREEMENT.
CUSTOMER agrees to hold FREESCALE harmless against any and all claims demands 
or actions by anyone on account of any damage, or injury, whether commercial,
contractual, or tortuous, rising directly or indirectly as a result 
of the advise or assistance supplied CUSTOMER in connection with product, 
services or goods supplied under this Agreement.
********************************************************************************
* File:             main.c
* Owner:            stnp002
* Version:          0.7
* Date:             Mar-07-2012
* Classification:   General Business Information
* Brief:            FlexPWM CTU ADC synchronization
					based on pin toggle stationery for MPC5675KEVB
********************************************************************************
* Detailed Description: 
*
* This example shows synchronization between FlexPWM, CTU and ADC modules.
* The FlexPWM Submodule 0 is initialized to generate PWM signal, and rising edge 
* of PWM B0 signal is used to generate trigger signal for CTU module. The CTU module 
* sends one command to ADC. Single conversion mode is used, and ADC0 ch0 
* is sampled. The conversion result is used to modify PWM B0 rising egde position within
* the ADC EOC interrupt.
*
* ------------------------------------------------------------------------------
* Test HW:  MPC5675KEVB, PPC5675KFMMSJ, DPM mode
* Target :  RAM
* Fsys:     180 MHz PLL.
*
* EVB connection: 
*
* J60.4 - B[7] .. ADC0 AN[0] input
* J65.10 - C[14] .. CTU0 EXT TRG output signal
* J39.1 - D[0] .. GPIO output, used to show ADC interrupt routine execution
* J8.20 -  FlexPWM B[0] output 
* J8.10 -  FlexPWM A[0] output
*
* connect Trimmer RV1 to ADC0_AN0 to change position of PWM B0 rising edge, place J38 jumper
* 
********************************************************************************
Revision History:
0.1     Aug-16-2011     b08110  Initial Version
0.2     Sep-02-2011     b08110  Fixed PLL0 freq and CLKOUT freq settings
0.3     Sep-03-2011     b08110  PIT3 1s interrupt to toggle LED1.
0.4     Oct-19-2011     b08110  Added clear NCF 12 (SAFE mode after reset
                                due to NCF 12 timeout. STCU shadow flash
                                instructions per AN4322.)
0.5     Feb-16-2012     b08110  Fixed peri set1 frequency.
                                per RM, max for this clock is 45 MHz.
                                thus, we must set MC_CGM.SC_DC1 = 0x81;
                                - divide system clock (90 MHz) by 2
0.6     Mar-07-2012     b08110  Mode Entry chnage of the system clock source
                                from IRC16 to PLL0 done with maximal ODF.
                                After change, the ODF is stepped to reach target
                                system clock frequency (ODF can be changed
                                on MPC5675K while running system clock from it).
                                This is to prevent VDD1.2 low voltage destructive
                                reset on the EVB at the time when the system
                                clock is switched from 16M IRC to PLL0.
0.7     Dec-02-2013     stnp002 add PWM, ADC, CTU related code                               

*******************************************************************************/
#include "MPC5675K.h"

/*******************************************************************************
* External objects
*******************************************************************************/
/* Prototype for second core startup */
extern void __start_p1();

/*******************************************************************************
* Global variables
*******************************************************************************/

/*******************************************************************************
* Constants and macros
*******************************************************************************/
#define FCCU_NCFK_KEY 0xAB3498FE
#define FCCU_CFK_KEY  0x618B7A50

/*******************************************************************************
* Local types
*******************************************************************************/

/*******************************************************************************
* Local function prototypes
*******************************************************************************/
static void InitHW(void);
static void InitSysclk(void);
static void InitGPIO(void);
static void DisableWatchdog(void);
static void InitModesAndClks(void);
static void InitPeriClkGen(void);

static void ClearFails(void);
static void ClearCF(void);
static void ClearNCF(void);

static void FlexPWM_Init(void);
static void CTU_Init(void);
static void ADC_Init(void);
static void CTU_ADC_ISR(void);

/*******************************************************************************
Function Name : InitHW
Engineer      : b08110
Date          : Feb-18-2010
Parameters    : NONE
Modifies      : NONE
Returns       : NONE
Notes         : initialization of the hw for the purposes of this example
Issues        : NONE
*******************************************************************************/
static void InitHW(void)
{
    DisableWatchdog();
    ClearFails();    
    InitSysclk();    
    InitGPIO();    
}

/*******************************************************************************
Function Name : InitSysclk
Engineer      : b08110
Date          : Feb-25-2010
Parameters    : NONE
Modifies      : NONE
Returns       : NONE
Notes         : NONE
Issues        : NONE
*******************************************************************************/
static void InitSysclk(void)
{
    InitModesAndClks();
    InitPeriClkGen();
}

/*******************************************************************************
Function Name : InitModesAndClks
Engineer      : b08110
Date          : Mar-07-2012
Parameters    : NONE
Modifies      : NONE
Returns       : NONE
Notes         : - DRUN->RUN1->RUN0->RUN1
                - enable CLKOUT
Issues        : NONE
*******************************************************************************/
static void InitModesAndClks(void) 
{
    int32_t cnt = 0;
    
    ME.MER.R = 0x0000003D;        /* Enable DRUN, RUN0, RUN1 SAFE, RESET modes */
    
    /* Mode Transition to enter RUN1 mode: */    
    ME.RUN[1].R = 0x001F0030;       /* RUN1 cfg: 16MHzIRCON,OSC0ON,PLL0OFF,PLL1OFF,syclk=16MIRC */
    ME.MCTL.R = 0x50005AF0;         /* Enter RUN1 Mode & Key */        
    ME.MCTL.R = 0x5000A50F;         /* Enter RUN1 Mode & Inverted Key */
    while(0 == ME.GS.B.S_XOSC) {};               /* Wait for mode entry to complete */
    while(1 == ME.GS.B.S_MTRANS) {}    /* Wait for mode transition to complete */
    while(5 != ME.GS.B.S_CURRENTMODE) {};       /* Check RUN1 mode has been entered */ 
    
    
    /* Initialize PLL before turning it on: */
    /* fsys = fcrystal*ndiv/idf/odf */
    /* fvco must be from 256 MHz to 512 MHz */
    /* we want fsys = 180 MHz. fvco = fsys*odf = 180 MHz * 2 = 360 MHz */
    /* fsys =  40*36/4/2 = 180 MHz */

    CGM.AC3_SC.R = 0x01000000; /* Select Xosc as PLL0 source clock */
    CGM.AC4_SC.R = 0x01000000; /* Select Xosc as PLL1 source clock */

#if 1
    /* 180 MHz */
    CGM.FMPLL[0].CR.B.IDF = 0x3;    /* FMPLL0 IDF=3 --> divide by 4 */
    CGM.FMPLL[0].CR.B.ODF = 0x0;    /* FMPLL0 ODF=0 --> divide by 2*/
    CGM.FMPLL[0].CR.B.NDIV = 36;    /* FMPLL0 NDIV=36 --> divide by 36 */
  
    /* 120 MHz */
    CGM.FMPLL[1].CR.B.IDF = 0x5;    /* FMPLL0 IDF=5 --> divide by 8 */
    CGM.FMPLL[1].CR.B.ODF = 0x1;    /* FMPLL0 ODF=1 --> divide by 4*/
    CGM.FMPLL[1].CR.B.NDIV = 72;    /* FMPLL0 NDIV=72 --> divide by 64 */  
#else
    /* 120 MHz */
    CGM.FMPLL[0].CR.B.IDF = 0x5;    /* FMPLL0 IDF=5 --> divide by 6 */
    CGM.FMPLL[0].CR.B.ODF = 0x1;    /* FMPLL0 ODF=1 --> divide by 4*/
    CGM.FMPLL[0].CR.B.NDIV = 72;    /* FMPLL0 NDIV=72 --> divide by 72 */
    
    /* 80 MHz */
    CGM.FMPLL[1].CR.B.IDF = 0x7;    /* FMPLL0 IDF=7 --> divide by 8 */
    CGM.FMPLL[1].CR.B.ODF = 0x1;    /* FMPLL0 ODF=1 --> divide by 4*/
    CGM.FMPLL[1].CR.B.NDIV = 64;    /* FMPLL0 NDIV=72 --> divide by 64 */
#endif

    /* enable progressive clock switching */ 
    /* PHI_PCS output only on PLL_0 */
    CGM.FMPLL[0].CR.B.EN_PLL_SW = 1; 	
        
    ME.RUNPC[0].R = 0x00000000; /* peripheral frozen in all modes */
    ME.RUNPC[1].R = 0x000000FE; /* enable peripheral run in all modes */
    ME.LPPC[0].R = 0x00000000;  /* disable peripheral run in LP modes */
    
    /* select ME_RUN_PC1 configuration */
    /* for the peripheral modules being used */
    ME.PCTL[32].R = 0x01; /* ADC0 */
    ME.PCTL[35].R = 0x01; /* CTU0 */
    ME.PCTL[41].R = 0x01; /* FlexPWM0 */
        
    /* peri set 0 disable, per set1 disable */
    CGM.SC_DC0.R = 0x00000000;
    CGM.AC0_DC0.R = 0x00000000;  /* disable Motor Control clk div by 1 */    
    CGM.AC1_DC0.R = 0x00000000;  /* disable aux clk 1 div by 1 */    
    CGM.AC2_DC0.R = 0x00000000;  /* disable aux clk 2 div by 1 */

    /* Mode Transition to enter RUN0 mode: */
    ME.RUN[0].R = 0x001F0070;       /* RUN0 cfg: 16MHzIRCON,OSC0ON,PLL0ON,PLL1off,syclk=16M IRC */    
    ME.MCTL.R = 0x40005AF0;         /* Enter RUN0 Mode & Key */
    ME.MCTL.R = 0x4000A50F;         /* Enter RUN0 Mode & Inverted Key */      
    while (1 == ME.GS.B.S_MTRANS) {}    /* Wait for mode transition to complete */
    while(4 != ME.GS.B.S_CURRENTMODE) {};       /* Check RUN0 mode has been entered */
    
    /* make sure the SRAM wait states are configured for fsys */
    ECSM_0.MUDCR.R = 0x40000000; 
    ECSM_1.MUDCR.R = 0x40000000;
    
    /* max divider when switching fsys from IRC to PLL0 */
    CGM.FMPLL[0].CR.B.ODF = 0x3; /* FMPLL0 ODF=3 --> divide by 16*/    
    
    /* Mode Transition to enter RUN1 mode: */    
    ME.RUN[1].R = 0x001F0074;       /* RUN1 cfg: 16MHzIRCON,OSC0ON,PLL0ON,PLL1off,syclk=PLL0 */            
    ME.MCTL.R = 0x50005AF0;         /* Enter RUN1 Mode & Key */        
    ME.MCTL.R = 0x5000A50F;         /* Enter RUN1 Mode & Inverted Key */
    while(1 == ME.GS.B.S_MTRANS) {}    /* Wait for mode transition to complete */
    while(5 != ME.GS.B.S_CURRENTMODE) {};       /* Check RUN1 mode has been entered */
    
    CGM.FMPLL[0].CR.B.ODF = 0x2; /* FMPLL0 ODF=2 --> divide by 8*/
    CGM.FMPLL[0].CR.B.ODF = 0x1; /* FMPLL0 ODF=1 --> divide by 4*/
    /* now manually tested delay for the VDD reg to stabilize .... */
    /* otherwise on the EVB, low voltage 1.2 VDD destructive reset might occur */    
    for(cnt=0;cnt<275;cnt++) {} 
    CGM.FMPLL[0].CR.B.ODF = 0x0; /* FMPLL0 ODF=0 --> divide by 2*/

#if 1
    /* Mode Transition to enter RUN1 mode: */    
    ME.RUN[1].R = 0x001F00F4;       /* RUN1 cfg: 16MHzIRCON,OSC0ON,PLL0ON,PLL1ON,syclk=PLL0 */            
    ME.MCTL.R = 0x50005AF0;         /* Enter RUN1 Mode & Key */        
    ME.MCTL.R = 0x5000A50F;         /* Enter RUN1 Mode & Inverted Key */
    while(1 == ME.GS.B.S_MTRANS) {}    /* Wait for mode transition to complete */
    while(5 != ME.GS.B.S_CURRENTMODE) {};       /* Check RUN1 mode has been entered */
#endif
    
    /* peri set 0 divide by 1, per set1 divide by 2 */    
    CGM.SC_DC0.R = 0x80810000;    
    
    /* set CLKOUT divider of 4 */
    CGM.OCDS_SC.R = 0x32000000; /* div by 4, system FMPLL */ 
    CGM.OC_EN.B.EN = 1; 	    /* enable CLKOUT signal */
    
    /* enable CLK_OUT on ball E20 */
    /* ALT1 - PCR[233] - PA = 0b01 */
    SIU.PCR[233].R = 0x0600;
}

/*******************************************************************************
Function Name : InitPeriClkGen
Engineer      : b08110
Date          : Oct-19-2011
Parameters    : NONE
Modifies      : NONE
Returns       : NONE
Notes         : - Enable auxiliary clock 1-2. from PLL0
Issues        : NONE
*******************************************************************************/
static void InitPeriClkGen(void) 
{
    CGM.AC0_SC.R = 0x05000000;  /* Select PLL1 for Motor Control Clock  */
    CGM.AC0_DC0.R = 0x80800000;  /* Enable Motor Control clk div by 1 */
    CGM.AC1_SC.R = 0x04000000;  /* Select PLL0 for aux clk 1  */
    CGM.AC1_DC0.R = 0x80000000;  /* Enable aux clk 1 div by 1 */
    CGM.AC2_SC.R = 0x04000000;  /* Select PLL0 for aux clk 2  */
    CGM.AC2_DC0.R = 0x80000000;  /* Enable aux clk 2 div by 1 */
}

/*******************************************************************************
Function Name : DisableWatchdog
Engineer      : b08110
Date          : Jun-23-2010
Parameters    : NONE
Modifies      : NONE
Returns       : NONE
Notes         : - disable SWT functionality
Issues        : NONE
*******************************************************************************/
void DisableWatchdog(void) 
{
    SWT_0.SR.R = 0x0000c520;     /* Write keys to clear soft lock bit */
    SWT_0.SR.R = 0x0000d928; 
    SWT_0.CR.R = 0x8000010A;     /* Clear watchdog enable (WEN) */
    
    SWT_1.SR.R = 0x0000c520;     /* Write keys to clear soft lock bit */
    SWT_1.SR.R = 0x0000d928; 
    SWT_1.CR.R = 0x8000010A;     /* Clear watchdog enable (WEN) */
    
    /* e200 Core Watchdog Timer */
    asm
    {
        li  r3, 0
        mtspr   340, r3
    }
}

/*******************************************************************************
Function Name : ClearNCF
Engineer      : b08110
Date          : Oct-19-2011
Parameters    : NONE
Modifies      : NONE
Returns       : NONE
Notes         : Clear NCF 12 only.
Issues        : NONE
*******************************************************************************/
static void ClearNCF(void)
{
    uint32_t i,b[4];
    for(i=0;i<1;i++)
    {
        FCCU.NCFK.R = FCCU_NCFK_KEY;
        FCCU.NCFS0.R = 0x00001000;  /* clear NCF 12 */
        while(FCCU.CTRL.B.OPS != 0x3)
        {
        
        };              /* wait for the completion of the operation */
        b[i]=FCCU.NCFS0.R;
    }
}

/*******************************************************************************
Function Name : ClearCF
Engineer      : b08110
Date          : Oct-19-2011
Parameters    : NONE
Modifies      : NONE
Returns       : NONE
Notes         : NONE
Issues        : NONE
*******************************************************************************/
static void ClearCF(void)
{
    uint32_t i,a[4];
    for(i=0;i<2;i++)
    {
        FCCU.CFK.R = FCCU_CFK_KEY;
   
        FCCU.CFS[i].R = 0xFFFFFFFF;
  
        while(FCCU.CTRL.B.OPS != 0x3)
        {
            
        };      /* wait for the completion of the operation */

        a[i]=FCCU.CFS[i].R;
    }
}

/*******************************************************************************
Function Name : ClearFails
Engineer      : b08110
Date          : Apr-08-2011
Parameters    : NONE
Modifies      : NONE
Returns       : NONE
Notes         : NONE
Issues        : NONE
*******************************************************************************/
static void ClearFails(void)
{
    uint16_t reset_register;
    
    if(RGM.FES.B.F_FCCU_SAFE)
    {
        reset_register = RGM.FES.R;
        ME.IMTS.R = 0x00000001;
        /*ClearCF();*/
        ClearNCF();
        
        RGM.FES.R = 0x0080; /* clear SAFE mode flag */
        /*RGM.DES.R = 0xFFFF;*/
        
        /* re-enter DRUN */
        ME.MCTL.R = 0x30005AF0; /* Enter DRUN Mode & Key */        
        ME.MCTL.R = 0x3000A50F; /* Enter DRUN Mode & Inverted Key */
    }
}

/*******************************************************************************
Function Name : InitGPIO
Engineer      : b08110
Date          : Feb-08-2010
Parameters    : NONE
Modifies      : NONE
Returns       : NONE
Notes         : 
Issues        : NONE
*******************************************************************************/
static void InitGPIO(void)
{
    /* ball P2 for User LED0 */    	 	
 	SIU.GPDO[62].R = 1;
        
    SIU.PCR[62].B.PA = 0x00; 
        
    SIU.PCR[62].B.OBE = 0x01;    
    
    SIU.PCR[23].R = 0x2000; // B[7] pin as ADC0 AN[0] input
    SIU.PCR[46].R = 0x0A04; // C[14] pin as CTU0 EXT TRG output
       
    SIU.PCR[58].R = 0x0600; // GPIO[58] pin as FlexPWM A[0] output 
    SIU.PCR[59].R = 0x0600; // GPIO[59] pin as FlexPWM B[0] output
    SIU.PCR[57].R = 0x0600; // GPIO[57] pin as FlexPWM X[0] output
    
}


/*******************************************************************************
Function Name : FlexPWM_Init
Engineer      : stnp002
Date          : Dec-2-2013
Parameters    : NONE
Modifies      : NONE
Returns       : NONE
Notes         : FlexPWM initialization
Issues        : NONE
*******************************************************************************/
static void FlexPWM_Init(void)
{
	/* Submodule 0 Initialisation */
    FLEXPWM_0.OUTEN.R           = 0x110;	// enable A and B outputs on submodule 0

    FLEXPWM_0.SUB[0].CTRL1.R     = 0x0400;	// full cycle reload, every opportunity,IPBus/1, double switching enable
    FLEXPWM_0.SUB[0].CTRL2.R    = 0x2000;
    FLEXPWM_0.SUB[0].TCTRL.R    = 0x0010;	// VAL4 reg determine OUT_TRIG0 generation
    FLEXPWM_0.SUB[0].DTCNT0.R   = 0x0033;	// deadtime values
    FLEXPWM_0.SUB[0].DTCNT1.R   = 0x0033;
    FLEXPWM_0.SUB[0].VAL[1].R   = 0x1770;	// PWM modulo 
    FLEXPWM_0.SUB[0].VAL[2].R   = 0x0000;   // PWM A0 rising edge
    FLEXPWM_0.SUB[0].VAL[3].R   = 0x0bb8;	// PWM A0 falling edge
    FLEXPWM_0.SUB[0].VAL[4].R   = 0x1000;	// PWM B0 rising edge
    FLEXPWM_0.SUB[0].VAL[5].R   = 0x1200;	// PWM B0 falling edge
    FLEXPWM_0.SUB[0].DISMAP.R   = 0x0000;	// disable fault pin condition

    FLEXPWM_0.MCTRL.R = 0x0100;		// Submodule 0 PWM generator enabled

    FLEXPWM_0.MCTRL.B.LDOK = 0x1;	/* Load the PRSC bits of CTRL1 and the INIT, and VALx registers 
    								   into a set of buffers */
}

/*******************************************************************************
Function Name : InitCTU
Engineer      : stnp002
Date          : Dec-2-2013
Parameters    : NONE
Modifies      : NONE
Returns       : NONE
Notes         : CTU initialization
Issues        : NONE
*******************************************************************************/
static void CTU_Init(void)
{
	/* Enable input */
    CTU_0.TGSISR.R              = 0x000000004; // Enable PWM B0 rising edge trigger; I1_RE

    /* Compare registers */
    CTU_0.TCR[0].R              = 0x0000; // For triggering conversion  
										  // determines delay between external trigger and triggering ADC sequence*/
    /* Counter */
    CTU_0.TGSCCR.R              = 0x1770;
    CTU_0.TGSCRR.R              = 0x0000;
    CTU_0.TGSCR.R               = 0x0000;

	

    /* Enable triggers */
    CTU_0.THCR1.R               = 0x00000061; // Enable Triger 0, enable external trigger signal and set to ADC /

    /* Set ADC messages */
	CTU_0.CLR[0].R              = 0x4000; // first command in command sequence, sample CH0 */
	CTU_0.CLR[1].R              = 0x4000; // dummy command to stop previous command sequence, 
    									  // first command of second sequence which is not started

    CTU_0.CTUCR.R               = 0x0003; /* General reload and TGS input selection reload */
}

/*******************************************************************************
Function Name : InitADC
Engineer      : stnp002
Date          : Dec-2-2013
Parameters    : NONE
Modifies      : NONE
Returns       : NONE
Notes         : ADC initialization
Issues        : NONE
*******************************************************************************/
static void ADC_Init(void)
{

    //------------------------
    // ADC 0 initialization
    //------------------------
    ADC_0.MCR.R	= 0x80020000;		// enable result overwriting, enable CTU triggered injected conversion
    ADC_0.CTR0.R  = 0x00008618;	    // Conversion times configuration for min sample time @ 60 MHz
	ADC_0.IMR.R = 0x00000010;		// enable EOCTU interrupt
	ADC_0.CIMR0.R = 0x00000001;		// enable interrupt from channel 0


}

/*******************************************************************************
Function Name : CTU_ADC_ISR
Engineer      : stnp002
Date          : Dec-2-2013
Parameters    : NONE
Modifies      : NONE
Returns       : NONE
Notes         : CTU_ADC command interrupt routine
Issues        : NONE
*******************************************************************************/
static void CTU_ADC_ISR(void)
{

    SIU.GPDO[62].R = 1; // LED ON
     
    /* update FlexPWM B0 rising edge position based on ADC1 ch0 input value */
    FLEXPWM_0.SUB[0].VAL[4].R   = (uint16_t)(0x1000 - ADC_0.CDR[0].B.CDATA) ;
    FLEXPWM_0.MCTRL.B.LDOK = 0x1;

    ADC_0.ISR.R = 0x00000010;	// clear ADC EOCTU flag
 
    //CTU_0.CTUCR.B.GRE  = 1;		// CTU General Reload Enable, not needed as no CTU buffered 
    								// registers are updated
    
    SIU.GPDO[62].R = 0;		// LED OFF
}


/*******************************************************************************
* Global functions
*******************************************************************************/

int32_t main(void) 
{
    volatile unsigned short i = 0;
    
    /* Initialize the MCU */
    InitHW();
	
	CTU_Init();
  	ADC_Init();
  
  	INTC_InstallINTCInterruptHandler(CTU_ADC_ISR, 62,2); /* install an interrupt handler (ADC_EOC) 
  														  for a given interrupt vector (62) and 
  														  set the interrupt priority (2) */
  
  	INTC_0.CPR_PRC0.R = 0;	/* Lower INTC's current priority, by default highest, e.g 15, is used */
	
	FlexPWM_Init();
	
	FLEXPWM_0.MCTRL.B.RUN = 0x1;		// Submodule 0 PWM generator enabled
  
#if 0
    /* Keep the 2nd core in reset for this applicaton */
  
    /* Start the second core, VLE mode*/
    SSCM.DPMBOOT.R = (unsigned long)__start_p1 + 0x00000002; 
    SSCM.DPMKEY.R = 0x00005AF0;
    SSCM.DPMKEY.R = 0x0000A50F;
#endif

 
    
  

    /* Loop forever */
    for (;;) 
    {
		i++;
    }
}



